###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: photos.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 07 Dec 2016
#   Purpose: Handle Photos, CuratedPhotos, RandomPhotos, and Photo
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
from .liblogging import logger
from .unpage import UnsplashPage
from .unobject import UnsplashObject


class Photos(UnsplashPage):
    def __init__(self, api_key, url='/photos', **kwargs):
        valid_options = ['page', 'per_page', 'order_by']
        super(Photos, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Photo(api_key=self.api_key, source=entry)


class CuratedPhotos(UnsplashPage):
    # TODO: 2019 this endpoint is deprecated
    def __init__(self, api_key, url='/photos/curated', **kwargs):
        logger.warning('This endpoint has been deprecated by the Unsplash API')
        valid_options = ['page', 'per_page', 'order_by']
        super(CuratedPhotos, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Photo(api_key=self.api_key, source=entry)


class RandomPhotos(UnsplashPage):
    # TODO: update documentation to reflect this at file
    # TODO: docs/source/classes/class_pyunsplash.rst
    #
    def __init__(self, api_key, url='/photos/random', **kwargs):
        valid_options = ['collections', 'featured', 'username', 'query', 'orientation', 'count']
        super(RandomPhotos, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Photo(api_key=self.api_key, source=entry)


class Photo(UnsplashObject):
    def __init__(self, api_key, source, **kwargs):
        valid_options = ['w', 'h', 'rect']
        # simplify for source=photoid
        if isinstance(source, str) and not source.startswith(self._api_root):
            # assume it's a photoid
            source = "{}/photos/{}".format(self._api_root, source)
        super(Photo, self).__init__(api_key=api_key, source=source, valid_options=valid_options, **kwargs)

    @property
    def link_html(self):
        return self.links.get('html', None)

    @property
    def stats(self):
        url = '{}/stats'.format(self.url)
        r = self._agent.get(url)
        if r.status_code == 200:
            return r.json()
        else:
            logger.error('Failed stats retrieval for %s with status code %s', url, r.status_code)
            return None

    @property
    def link_download(self):
        return self.links.get('download', None)

    @property
    def link_download_location(self):
        return self.links.get('download_location', None)




###############################################################################
#
#      File: pyunsplash.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 08 Dec 2016
#   Purpose: Main class
#
#   Comment:
#
###############################################################################
from .src.liblogging import logger
from .src.collections import Collections, FeaturedCollections, CuratedCollections
from .src.photos import Photos, CuratedPhotos, RandomPhotos
from .src.search import Search
from .src.stats import Stats
from .src.users import User
from .src.settings import LIB_NAME


class PyUnsplash(object):
    logger_name = LIB_NAME

    def __init__(self, api_key):
        self._api_key = api_key

    # TODO: oAuth

    def user(self, source, **kwargs):
        return User(api_key=self._api_key, source=source, **kwargs)

    def collections(self, type_='generic', **kwargs):
        lookup = {'curated': CuratedCollections,  # 2019: DEPRECATED
                  'generic': Collections,
                  'featured': FeaturedCollections}

        if type_ in lookup:
            f = lookup.get(type_)
            return f(api_key=self._api_key, **kwargs)

        logger.debug('No collections object to return')
        return None

    def photos(self, type_='generic', **kwargs):
        lookup = {'curated': CuratedPhotos,  # 2019: DEPRECATED
                  'generic': Photos,
                  'random': RandomPhotos}

        if type_ in lookup:
            f = lookup.get(type_)
            return f(api_key=self._api_key, **kwargs)

        logger.debug('No photos object to return')
        return None

    def search(self, type_, **kwargs):
        return Search(api_key=self._api_key, where=type_, **kwargs)

    def stats(self):
        return Stats(api_key=self._api_key)



###############################################################################
#
#      File: resource_download.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 14 Dec 2016
#   Purpose: Download and save resources for unit testing
#
#   Comment: The APPLICATION ID is taken from OS ENV
#
###############################################################################
from pyunsplash.src.settings import API_ROOT
import requests
import os
import json
import time

api_key = os.environ['APPLICATION_ID']

req_headers = {
    'Accept-Version': 'v1',
    'Authorization': 'Client-ID %s' % api_key,
}


def _save_content(sub_url):
    url = '{}{}'.format(API_ROOT, sub_url)
    r = requests.get(url=url, headers=req_headers)

    out = {
        'url': sub_url,
        'body': r.json(),
        'headers': dict(r.headers),
        'status_code': r.status_code,
        'last_update': time.ctime()
    }

    # save to file
    filename = sub_url.replace('/', '_').replace('?', '_').replace('=', '_')
    print(os.getcwd())
    filepath = os.sep.join(['.', 'pyunsplash', 'tests', 'resources', 'resource_{}.json'.format(filename)])
    with open(filepath, 'w') as f:
        f.write(json.dumps(out))


def get_resources_collections():
    for sub_url in ['/collections?page=2', '/collections/featured?page=2', '/collections/curated?page=2']:
        _save_content(sub_url)


def get_resources_photos():
    for sub_url in ['/photos?page=2', '/photos/curated?page=2', '/photos/random?count=2']:
        _save_content(sub_url)


def get_resources_stats():
    for sub_url in ['/stats/total']:
        _save_content(sub_url)


def get_resources_users():
    for sub_url in ['/users/salvoventura', '/users/salvoventura/statistics']:
        _save_content(sub_url)


def get_resources_search():
    for sub_url in ['/search/photos?query=blue', '/search/collections?query=tree', '/search/users?query=ventura']:
        _save_content(sub_url)


def main():
    get_resources_collections()
    get_resources_photos()
    get_resources_stats()
    get_resources_users()
    get_resources_search()


if __name__ == '__main__':
    main()



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: rest.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 12 Oct 2016
#   Purpose: Handler for REST API interaction
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
import requests
from .liblogging import logger


class Rest(object):
    def __init__(self, api_key):
        self._api_key = api_key
        self._req_headers = {
            'Accept-Version': 'v1',
            'Authorization': 'Client-ID %s' % api_key,
        }
        self._status_code = None
        self._body = None
        self._headers = None

    @staticmethod
    def _query_parameters(query_parameters):
        """
        Prepare the query parameter string to be appended to the url

        :param query_parameters: dictionary of query parameters and their values
        :return:
        """
        logger.debug('calling _query_parameters(%s)', query_parameters)
        if query_parameters is None:
            return ''

        tmp = []
        for k, v in query_parameters.items():
            if v is None:
                continue
            if k == 'rect':
                tmp.append('{}={}'.format(k, ','.join(map(str, v.values()))))
            else:
                tmp.append('{}={}'.format(k, v))
        return '&'.join(tmp)

    @property
    def status_code(self):
        return self._status_code

    @property
    def body(self):
        return self._body

    @property
    def headers(self):
        return self._headers

    @property
    def links(self):
        if hasattr(self.body, 'links'):
            return self.body.get('links')
        return {}

    def get(self, url, query_params=None):
        logger.debug('calling rest get %s %s', url, query_params)
        _url = url
        if query_params:
            if url.find('?') > 0:
                _url = '&'.join([url, self._query_parameters(query_parameters=query_params)])
            else:
                _url = '?'.join([url, self._query_parameters(query_parameters=query_params)])

        logger.debug('rest get %s', _url)
        _r = requests.get(_url, headers=self._req_headers, allow_redirects=True)

        try:
            self._status_code = _r.status_code
            self._headers = _r.headers
            self._body = _r.json()
            if self._status_code != requests.codes.ok:
                logger.error(
                    'HTTP status %s: %s', self._status_code, self._body.get('errors', ['No error message'])
                )

        except ValueError as e:
            # If you get a 403, the body is NOT json
            # Good for logging, but can't really protect much, as anyway the
            # object up is likely going to run into its own exception for unexpected
            # result, and will make debugging this so much more difficult.
            # With some use, might get a better/smarter way of dealing with this.
            logger.error('EXCEPTION: %s', e)
            if self._status_code != requests.codes.ok:
                logger.error('HTTP EXC status %s: %s', self._status_code, _r.text)
            raise

        logger.debug('rest rsp status %s body %s headers %s', self._status_code, self._body, self._headers)
        return _r

    def put(self, url, body):
        pass

    def post(self, url, body):
        pass

    def delete(self, url):
        pass
